@@ -26,7 +26,7 @@ from firewall.core.fw_transaction import FirewallTransaction
 from firewall.core.io.policy import Policy
 from firewall.core.logger import log
 from firewall.core.rich import Rich_Service, Rich_Port, Rich_Protocol, Rich_SourcePort, Rich_ForwardPort, \
-                               Rich_IcmpBlock, Rich_IcmpType, Rich_Masquerade, Rich_Mark
+                               Rich_IcmpBlock, Rich_IcmpType, Rich_Masquerade, Rich_Mark, Rich_Tcp_Mss_Clamp
 from firewall.functions import checkIPnMask, checkIP6nMask, check_mac
 from firewall import errors
 from firewall.errors import FirewallError
@@ -110,7 +110,7 @@ class FirewallZone(object):
                 # any zone --> zone
                 setattr(p_obj, setting, copy.deepcopy(getattr(z_obj, setting)))
             elif fromZone == z_obj.name and toZone == "ANY" and \
-                 setting in ["icmp_blocks", "forward_ports"]:
+                 setting in ["forward_ports"]:
                 # zone --> any zone
                 setattr(p_obj, setting, copy.deepcopy(getattr(z_obj, setting)))
             elif setting in ["rules"]:
@@ -134,7 +134,7 @@ class FirewallZone(object):
         # Create policy objects, will need many:
         #   - (zone --> HOST) - ports, service, etc
         #   - (any zone --> zone) - masquerade
-        #   - (zone --> any zone) - ICMP block, icmp block inversion
+        #   - (zone --> any zone)
         #       - also includes forward-ports because it works on (nat,
         #       PREROUTING) and therefore applies to redirects to the local
         #       host or dnat to a different host.
@@ -208,13 +208,13 @@ class FirewallZone(object):
 
         (zone, _chain) = x
         # derived from _get_table_chains_for_zone_dispatch()
-        if _chain in ["PREROUTING", "FORWARD_IN"]:
+        if _chain in ["PREROUTING", "FORWARD"]:
             fromZone = zone
             toZone = "ANY"
         elif _chain in ["INPUT"]:
             fromZone = zone
             toZone = "HOST"
-        elif _chain in ["POSTROUTING", "FORWARD_OUT"]:
+        elif _chain in ["POSTROUTING"]:
             fromZone = "ANY"
             toZone = zone
         else:
@@ -441,9 +441,11 @@ class FirewallZone(object):
             raise FirewallError(errors.ZONE_ALREADY_SET,
                                 "'%s' already bound to '%s'" % (interface,
                                                                 zone))
-        if self.get_zone_of_interface(interface) is not None:
+        zoi = self.get_zone_of_interface(interface)
+        if zoi is not None:
             raise FirewallError(errors.ZONE_CONFLICT,
-                                "'%s' already bound to a zone" % interface)
+                                "'%s' already bound to '%s'" % (interface,
+                                                                 zoi))
 
         log.debug1("Setting zone of interface '%s' to '%s'" % (interface,
                                                                _zone))
@@ -849,15 +851,14 @@ class FirewallZone(object):
         if type(rule.action) == Rich_Mark:
             return [self.policy_name_from_zones(zone, "ANY")]
         elif type(rule.element) in [Rich_Service, Rich_Port, Rich_Protocol,
-                                    Rich_SourcePort]:
+                                    Rich_SourcePort, Rich_IcmpBlock, Rich_IcmpType]:
             return [self.policy_name_from_zones(zone, "HOST")]
-        elif type(rule.element) in [Rich_IcmpBlock, Rich_IcmpType]:
-            return [self.policy_name_from_zones(zone, "HOST"),
-                    self.policy_name_from_zones(zone, "ANY")]
         elif type(rule.element) in [Rich_ForwardPort]:
             return [self.policy_name_from_zones(zone, "ANY")]
         elif type(rule.element) in [Rich_Masquerade]:
             return [self.policy_name_from_zones("ANY", zone)]
+        elif type(rule.element) in [Rich_Tcp_Mss_Clamp]:
+            return [self.policy_name_from_zones(zone, "ANY")]
         elif rule.element is None:
             return [self.policy_name_from_zones(zone, "HOST")]
         else:
@@ -959,8 +960,6 @@ class FirewallZone(object):
         p_name = self.policy_name_from_zones(zone, "HOST")
         self._fw.policy.add_icmp_block(p_name, icmp, timeout, sender)
 
-        p_name = self.policy_name_from_zones(zone, "ANY")
-        self._fw.policy.add_icmp_block(p_name, icmp, timeout, sender)
         return zone
 
     def remove_icmp_block(self, zone, icmp):
@@ -968,31 +967,23 @@ class FirewallZone(object):
         p_name = self.policy_name_from_zones(zone, "HOST")
         self._fw.policy.remove_icmp_block(p_name, icmp)
 
-        p_name = self.policy_name_from_zones(zone, "ANY")
-        self._fw.policy.remove_icmp_block(p_name, icmp)
         return zone
 
     def query_icmp_block(self, zone, icmp):
         zone = self._fw.check_zone(zone)
         p_name_host = self.policy_name_from_zones(zone, "HOST")
-        p_name_fwd = self.policy_name_from_zones(zone, "ANY")
-        return self._fw.policy.query_icmp_block(p_name_host, icmp) and \
-               self._fw.policy.query_icmp_block(p_name_fwd, icmp)
+        return self._fw.policy.query_icmp_block(p_name_host, icmp)
 
     def list_icmp_blocks(self, zone):
         zone = self._fw.check_zone(zone)
         p_name_host = self.policy_name_from_zones(zone, "HOST")
-        p_name_fwd = self.policy_name_from_zones(zone, "ANY")
-        return sorted(set(self._fw.policy.list_icmp_blocks(p_name_host) +
-                          self._fw.policy.list_icmp_blocks(p_name_fwd)))
+        return sorted(set(self._fw.policy.list_icmp_blocks(p_name_host)))
 
     def add_icmp_block_inversion(self, zone, sender=None):
         zone = self._fw.check_zone(zone)
         p_name = self.policy_name_from_zones(zone, "HOST")
         self._fw.policy.add_icmp_block_inversion(p_name, sender)
 
-        p_name = self.policy_name_from_zones(zone, "ANY")
-        self._fw.policy.add_icmp_block_inversion(p_name, sender)
         return zone
 
     def _icmp_block_inversion(self, enable, zone, transaction):
@@ -1000,24 +991,17 @@ class FirewallZone(object):
         p_name = self.policy_name_from_zones(zone, "HOST")
         self._fw.policy._icmp_block_inversion(enable, p_name, transaction)
 
-        p_name = self.policy_name_from_zones(zone, "ANY")
-        self._fw.policy._icmp_block_inversion(enable, p_name, transaction)
-
     def remove_icmp_block_inversion(self, zone):
         zone = self._fw.check_zone(zone)
         p_name = self.policy_name_from_zones(zone, "HOST")
         self._fw.policy.remove_icmp_block_inversion(p_name)
 
-        p_name = self.policy_name_from_zones(zone, "ANY")
-        self._fw.policy.remove_icmp_block_inversion(p_name)
         return zone
 
     def query_icmp_block_inversion(self, zone):
         zone = self._fw.check_zone(zone)
         p_name_host = self.policy_name_from_zones(zone, "HOST")
-        p_name_fwd = self.policy_name_from_zones(zone, "ANY")
-        return self._fw.policy.query_icmp_block_inversion(p_name_host) and \
-               self._fw.policy.query_icmp_block_inversion(p_name_fwd)
+        return self._fw.policy.query_icmp_block_inversion(p_name_host)
 
     def _forward(self, enable, zone, transaction):
         p_name = self.policy_name_from_zones(zone, "ANY")
